#!/usr/bin/env python

import datetime
from dm.common.exceptions.invalidRequest import InvalidRequest
from dm.common.cli.dmCli import DmCli
from dm.common.utility.configurationManager import ConfigurationManager
from dm.common.utility.ldapUserManager import LdapUserManager
from dm.aps_user_db.api.apsUserDbApi import ApsUserDbApi
from dm.ds_web_service.api.dsRestApiFactory import DsRestApiFactory

class UpdateUsersFromApsDbCli(DmCli):
    def __init__(self):
        DmCli.__init__(self)

    def runCommand(self):
        self.parseArgs(usage="""
    dm-update-users-from-aps-db 

Description:
    Updates all users in DM LDAP and database using information from APS DB. 
        """)
        ConfigurationManager.getInstance().setConfigFile('/home/oxygen/SVESELI/Work/DM/etc/dm.conf')
        self.logger.debug('Starting sync process')
        apsUserDbApi = ApsUserDbApi()
        userRestApi = DsRestApiFactory.getUserRestApi()

        self.logger.debug('Retrieving APS users')
        apsUsers = apsUserDbApi.getApsUsers()
        self.logger.debug('Number of APS Users: %s' % len(apsUsers))
        self.logger.debug('Retrieving DM users')
        dmUsers = userRestApi.getUsers()
        self.logger.debug('Number of DM Users: %s' % len(dmUsers))
        dmUserBadgeMap = {}
        for dmUser in dmUsers:
            badge = dmUser.get('badge')
            if badge: 
                dmUserBadgeMap[badge] = dmUser
        self.logger.debug('Number of DM Users with badge: %s' % len(dmUserBadgeMap))

        nCreatedUsers = 0
        nUpdatedUsers = 0
        for apsUser in apsUsers:
            apsLastChangeDate = apsUser.get('lastChangeDate')

            badge = apsUser['badgeNumber']
            username = 'd%s' % badge
            firstName = apsUser.get('firstName')
            middleName = apsUser.get('middleName')
            lastName = apsUser.get('lastName')
            email = apsUser.get('email')
            isLocalUser = 0
            lastUpdate = str(apsLastChangeDate)
            globusUsername = None
            description = None
            password = None

            dmUser = dmUserBadgeMap.get(badge)
            if not dmUser:
                self.logger.debug('Creating user with badge %s: %s' % (badge, apsUser['name']))

                userRestApi.addUser(username, firstName, lastName, middleName, email, badge, globusUsername, description, password, isLocalUser, lastUpdate)
                nCreatedUsers += 1
            else:
                dmLastUpdate = dmUser.get('lastUpdate')
                if dmLastUpdate:
                    # need to convert DM last update time to datetime object
                    # datetime cannot handle time zone at the moment
                    timeZonePos = dmLastUpdate.rfind('-')
                    dmLastUpdate = dmLastUpdate[0:timeZonePos]
                    dmLastUpdate = datetime.datetime.strptime(dmLastUpdate, '%Y-%m-%d %H:%M:%S')
                if not dmLastUpdate or dmLastUpdate < apsLastChangeDate:
                    id = dmUser.get('id')
                    self.logger.debug('Modifying user with badge %s: %s (DM id: %s)' % (badge, apsUser['name'], id))
                    print apsUser
                    userRestApi.updateUser(id, username, firstName, lastName, middleName, email, badge, globusUsername, description, password, isLocalUser, lastUpdate)
                    nUpdatedUsers += 1
                else:
                    # User is up to date
                    pass
           
        self.logger.debug('Number of new DM users: %s' % (nCreatedUsers))
        self.logger.debug('Number of updated DM users: %s' % (nUpdatedUsers))
        self.logger.debug('Completed sync process')

        #ldapUserManager = LdapUserManager('ldaps://dmid-vm.xray.aps.anl.gov:636', 'uid=dmadmin,ou=People,o=aps.anl.gov,dc=aps,dc=anl,dc=gov', '/tmp/ldapPassword', userDnFormat='uid=%s,ou=DM,ou=People,o=aps.anl.gov,dc=aps,dc=anl,dc=gov', groupDnFormat='cn=%s,ou=DM,ou=Group,o=aps.anl.gov,dc=aps,dc=anl,dc=gov', minGidNumber=66000)
        #ldapUser = ldapUserManager.getUserInfo(username)
        #print 'Old LDAP User Info: ', ldapUser
        #attrDict = {}
        #attrDict['gecos'] = apsUser['name']
        #attrDict['userPassword'] = apsUser['passwordHashValue']
        #ldapUserManager.modifyUserInfo(username, attrDict)
        #ldapUser = ldapUserManager.getUserInfo(username)
        #print 'New LDAP User Info: ', ldapUser

#######################################################################
# Run command.
if __name__ == '__main__':
    cli = UpdateUsersFromApsDbCli()
    cli.run()
